package de.ozgcloud.eingang.common.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.LocalDate;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;

class VorgangNummerSupplierTest {

	@Spy
	@InjectMocks
	private VorgangNummerSupplier vorgangNummerSupplier;

	@Nested
	@DisplayName("Get Vorgang Nummer")
	class TestGetVorgangNummer {

		@Test
		void shouldCallAddPrefix() {
			vorgangNummerSupplier.get();

			verify(vorgangNummerSupplier).initWithPrefix();
		}

		@Test
		@DisplayName("should add random suffix of length 6")
		void shouldAddSuffix() {
			doReturn(new StringBuilder()).when(vorgangNummerSupplier).initWithPrefix();

			var result = vorgangNummerSupplier.get();

			assertThat(result).hasSize(6);
		}
	}

	@Nested
	class TestAddPrefix {

		@Test
		void shouldHaveSize() {
			var resultBuilder = vorgangNummerSupplier.initWithPrefix();

			assertThat(resultBuilder).hasSize(6);
		}

		@Test
		void shouldAddLastYearNumberFirst() {
			var lastYearNumber = "" + LocalDate.now().getYear() % 10;

			var resultBuilder = vorgangNummerSupplier.initWithPrefix();

			assertThat(resultBuilder.substring(0, 1)).isEqualTo(lastYearNumber);
		}

		@Test
		void shouldAddMonthValueSecond() {
			var monthValue = "%02d".formatted(LocalDate.now().getMonthValue());

			var resultBuilder = vorgangNummerSupplier.initWithPrefix();

			assertThat(resultBuilder.substring(1, 3)).isEqualTo(monthValue);
		}

		@Test
		void shouldAddDayValueThird() {
			var dayValue = "%02d".formatted(LocalDate.now().getDayOfMonth());

			var resultBuilder = vorgangNummerSupplier.initWithPrefix();

			assertThat(resultBuilder.substring(3, 5)).isEqualTo(dayValue);
		}

		@Test
		void shouldAddHyphenAtEnd() {
			var resultBuilder = vorgangNummerSupplier.initWithPrefix();

			assertThat(resultBuilder.charAt(5)).isEqualTo('-');
		}
	}

}