package de.ozgcloud.eingang.enterprise.entry;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.tuple.Pair;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;

@Mapper
public interface FormDataMapper {

	public static final String VALUE_KEY = "value";
	public static final String LABEL_KEY = "label";

	@Mapping(target = "antragsteller", ignore = true)
	@Mapping(target = "attachment", ignore = true)
	@Mapping(target = "attachments", ignore = true)
	@Mapping(target = "numberOfAttachments", ignore = true)
	@Mapping(target = "representation", ignore = true)
	@Mapping(target = "representations", ignore = true)
	@Mapping(target = "numberOfRepresentations", ignore = true)

	@Mapping(target = "id", ignore = true)
	@Mapping(target = "header", source = "control")

	@Mapping(target = "zustaendigeStelle", source = "control.zustaendigeStelle")
	FormData mapEntryData(EntryData entryData);

	@Mapping(target = "createdAt", ignore = true)
	@Mapping(target = "formEngineName", constant = "EnterpriseSoftware")
	@Mapping(target = "requestId", source = "transactionId")
	@Mapping(target = "sender", ignore = true) // TODO fill from authentication
	@Mapping(target = "serviceKonto", ignore = true)
	FormHeader mapHeader(EntryData.ControlData controlData);

	default ZustaendigeStelle fromId(String organisationsEinheitenId) {
		return ZustaendigeStelle.builder().organisationseinheitenId(organisationsEinheitenId).build();
	}

	default Map<String, Object> mapFormItems(List<EntryFormDataItem> items) {
		return items.stream().map(item -> Pair.of(item.getName(),
				item.isFormField() ? mapFormField((EntryFormDataField) item) : mapSubForm((EntryFormDataSubForm) item)))
				.collect(Collectors.toMap(Pair::getKey, Pair::getValue));
	}

	default Map<String, Object> mapFormField(EntryFormDataField field) {
		return Map.of(LABEL_KEY, field.getLabel(), VALUE_KEY, field.getValue());
	}

	default Map<String, Object> mapSubForm(EntryFormDataSubForm subForm) {
		return Map.of(LABEL_KEY, subForm.getLabel(), VALUE_KEY, mapFormItems(subForm.getFormItems()));
	}

}
