/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formcycle.common.protobuf;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.lang.reflect.Type;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Spy;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpOutputMessage;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.formcycle.common.protobuf.ProtobufMessageConverter;
import lombok.SneakyThrows;

class ProtobufMessageConverterTest {

	@Spy
	@InjectMocks
	private ProtobufMessageConverter converter;

	@Nested
	class TestRead {

		private HttpInputMessage inputMessage = mock(HttpInputMessage.class);

		@Test
		@SneakyThrows
		void shouldCallReadInternal() {
			doReturn(null).when(converter).readInternal(any(), any());

			converter.read(String.class, Object.class, inputMessage);

			verify(converter).readInternal(eq(String.class), same(inputMessage));
		}

		@Test
		@SneakyThrows
		void shouldThrowTechnicalException() {
			var mockType = buildMockType();

			assertThatThrownBy(() -> converter.read(mockType, Object.class, inputMessage)).isInstanceOf(TechnicalException.class);
		}
	}

	@Nested
	class TestReadInternal {
		@Test
		void shouldThrowTechnicalException() {
			assertThatThrownBy(() -> converter.readInternal(Object.class, mock(HttpInputMessage.class)))
					.isInstanceOf(TechnicalException.class);
		}
	}

	@Nested
	class TestWriteInternal {
		@Test
		void shouldThrowTechnicalException() {
			var mockType = buildMockType();

			assertThatThrownBy(() -> converter.writeInternal("test", mockType, mock(HttpOutputMessage.class)))
					.isInstanceOf(TechnicalException.class);
		}
	}

	private Type buildMockType() {
		var mockType = mock(Type.class);
		when(mockType.getTypeName()).thenReturn("quatsch");

		return mockType;
	}
}
