/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.IncomingFile;

@Component
class FormSolutionsRepresentationsMapper {

	public static final String PDF = "pdf";
	public static final String FILE_NAME_PDF_REP = "eingang.pdf";
	public static final String PDF_CONTENT_TYPE = MediaType.APPLICATION_PDF_VALUE;

	public static final String JSON = "json";
	public static final String FILE_NAME_JSON_REP = "form-data.json";
	public static final String JSON_CONTENT_TYPE = MediaType.APPLICATION_JSON_VALUE;

	public static final String TMP_FILE_PREFIX = "filecached-inputstream-fs";
	public static final String TMP_FILE_SUFFIX = ".ozg-cloud.tmp";

	List<IncomingFile> mapRepresentations(Map<String, Object> plainMap, File jsonFile) {
		List<IncomingFile> representations = new ArrayList<>();

		Optional.ofNullable((String) plainMap.get(PDF))
				.filter(StringUtils::isNoneEmpty)
				.ifPresent(data -> representations.add(buildPdfFile(data)));

		representations.add(buildJsonFile(jsonFile));

		return representations;
	}

	private IncomingFile buildJsonFile(File jsonFile) {
		return IncomingFile.builder()
				.file(jsonFile)
				.contentType(JSON_CONTENT_TYPE)
				.name(FILE_NAME_JSON_REP)
				.size(jsonFile.length())
				.build();
	}

	private IncomingFile buildPdfFile(String data) {
		File file = FormSolutionsFileMapperUtils.decodeBase64Content(data);

		return IncomingFile.builder()
				.file(file)
				.contentType(PDF_CONTENT_TYPE)
				.name(FILE_NAME_PDF_REP)
				.size(file.length())
				.build();
	}

}