/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.formsolutions;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.semantik.enginebased.FilesMapperHelper;

@Component
class FormSolutionsRequestMapper {
	static final TypeReference<Map<String, Object>> VALUE_TYPE_REF = new TypeReference<Map<String, Object>>() {
	};

	@Autowired
	private FormSolutionsAttachmentsMapper attachmentMapper;
	@Autowired
	private FormSolutionsRepresentationsMapper representationMapper;
	@Autowired
	private ObjectMapper objectMapper;

	public FormData map(File jsonFile) {
		var formData = buildFormData(jsonFile);

		return mapFiles(formData, jsonFile);
	}

	private FormData buildFormData(File jsonFile) {
		return FormData.builder().formData(mapFormData(jsonFile)).build();
	}

	Map<String, Object> mapFormData(File jsonFile) {
		try (var in = new FileInputStream(jsonFile)) {
			return objectMapper.readValue(in, VALUE_TYPE_REF);
		} catch (IOException e) {
			throw new TechnicalException("Error parsing JSON from FormSolutions-Server", e);
		}
	}

	FormData mapFiles(FormData formData, File jsonFile) {
		return FormDataUtils.from(formData)
				.put(FilesMapperHelper.FIELD_NAME_MAPPED_FILES, buildMappedFiles(formData, jsonFile))
				.remove(FormSolutionsAttachmentsMapper.ZIP)
				.remove(FormSolutionsRepresentationsMapper.PDF)
				.build();

	}

	private Map<String, Object> buildMappedFiles(FormData formData, File jsonFile) {
		return Map.of(
				FilesMapperHelper.ATTACHMENTS, attachmentMapper.mapAttachments(formData.getFormData()),
				FilesMapperHelper.REPRESENTATIONS, representationMapper.mapRepresentations(formData.getFormData(), jsonFile));
	}
}