package de.ozgcloud.eingang.intelliform;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.itvsh.kop.common.errorhandling.TechnicalException;

@Component
class JsonService {

	static final TypeReference<List<Map<String, Object>>> VALUE_TYPE_REF = new TypeReference<List<Map<String, Object>>>() {
	};

	@Autowired
	private ObjectMapper objectMapper;

	public List<Map<String, Object>> readAsListMap(String json) {
		return readValueSafety(json, VALUE_TYPE_REF);
	}

	private <T> T readValueSafety(String json, TypeReference<T> typeRef) {
		try {
			return objectMapper.readValue(json, typeRef);
		} catch (JsonProcessingException e) {
			throw new TechnicalException("Error parsing JSON", e);
		}
	}
}