/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.router;

import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import de.itvsh.ozg.pluto.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.itvsh.ozg.pluto.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;

@SpringBootTest(properties = {
		"grpc.client.pluto-kiel.address=static://127.0.0.1:9090"
})
class PlutoServerResolverITCase {

	@Autowired
	private PlutoServerResolver resolver;

	@Test
	void shouldReturnVorgangServiceBlockingStub() {
		var created = resolver
				.resolveVorgangServiceBlockingStubByOrganisationseinheitenId(Optional.of(PlutoListPropertiesTestFactory.ORGANISATIONSEINHEIT_ID));

		assertThat(created).isNotNull().isInstanceOf(VorgangServiceBlockingStub.class);
	}

	@Test
	void shouldReturnBinaryFileServiceStub() {
		var created = resolver
				.resolveBinaryFileServiceStubByOrganisationsEinheitId(Optional.of(PlutoListPropertiesTestFactory.ORGANISATIONSEINHEIT_ID));

		assertThat(created).isNotNull().isInstanceOf(BinaryFileServiceStub.class);
	}
}
