/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class FilesMapperHelper {

	public static final String REPRESENTATIONS = "parsedRepresentations";
	public static final String FIELD_NAME_MAPPED_FILES = "mappedFiles";
	public static final String ATTACHMENTS = "parsedAttachments";

	public static Optional<Map<String, Object>> getMappedFiles(FormData formData) {
		return Optional.ofNullable(formData).map(FormData::getFormData).map(map -> (Map<String, Object>) map.get(FIELD_NAME_MAPPED_FILES));
	}
	public static Optional<List<IncomingFileGroup>> getAttachedFileGroups(FormData formData) {
		return getMappedFiles(formData).map(mappedFiles -> (List<IncomingFileGroup>) mappedFiles.get(ATTACHMENTS));
	}

	public static Optional<List<IncomingFile>> getRepresentations(FormData formData) {
		return getMappedFiles(formData).map(mappedFiles -> (List<IncomingFile>) mappedFiles.get(REPRESENTATIONS));
	}

	public static int countAttachedFiles(Collection<IncomingFileGroup> fileGroups) {
		return fileGroups.stream().filter(Objects::nonNull).mapToInt(group -> group.getFiles().size()).sum();
	}

	public static FormData removeProcessedData(FormData formData) {
		return FormDataUtils.from(formData).remove(FIELD_NAME_MAPPED_FILES).build();
	}
}
