/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.semantik.enginebased.ServiceKontoBuildHelper;

@Component
class AfmHeaderMapper implements AfmEngineBasedMapper {

	static final String AFM_FORMENGINE_NAME = "AFM";

	static final String POSTFACH_NAME_ID = "nameid";

	static final String HEADER_FIELD = "header";

	static final String ID = "t:id";
	static final String TIMESTAMP = "t:timestamp";
	static final String FORM_ID = "t:form-id";
	static final String FORM = "t:form";
	static final String SENDER = "t:sender";

	@Autowired
	private ServiceKontoBuildHelper serviceKontoBuildHelper;

	@Override
	public FormData parseFormData(FormData formData) {
		var processed = formData.toBuilder().header(buildHeader(formData)).build();

		return removeMappedData(processed);
	}

	private FormHeader buildHeader(FormData formData) {
		var headerDataMap = getHeaderMap(formData);

		var formHeaderBuilder = FormHeader.builder()
				.requestId((String) headerDataMap.get(ID))
				.createdAt(getCreatedAt(headerDataMap))
				.formId((String) headerDataMap.get(FORM_ID))
				.formName((String) headerDataMap.get(FORM))
				.sender((String) headerDataMap.get(SENDER))
				.formEngineName(AFM_FORMENGINE_NAME)
				.build();

		Optional.ofNullable(getNameId(formData))
				.map(nameId -> serviceKontoBuildHelper.buildServiceKonto(nameId, formData))
				.ifPresent(formHeaderBuilder::setServiceKonto);

		return formHeaderBuilder;
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getHeaderMap(FormData formData) {
		return (Map<String, Object>) formData.getFormData().get(HEADER_FIELD);
	}

	private ZonedDateTime getCreatedAt(Map<String, Object> headerDataMap) {
		return ZonedDateTime.parse((String) headerDataMap.get(TIMESTAMP), DateTimeFormatter.ISO_OFFSET_DATE_TIME);
	}

	private String getNameId(FormData formData) {
		return (String) formData.getFormData().get(POSTFACH_NAME_ID);
	}

	private FormData removeMappedData(FormData formData) {
		return FormDataUtils.from(formData)
				.remove(HEADER_FIELD)
				.remove(ServiceKontoBuildHelper.REST_RESPONSE_NAME)
				.build();
	}
}