/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;

@Component
class AfmZustaendigeStelleMapper implements AfmEngineBasedMapper {

	public static final String ZUSTAENDIGESTELLE = "zustaendigestelle";

	public static final String EMAIL = "emailadresse";
	public static final String ORGANISATIONSEINHEITEN_ID = "OrganisationseinheitenID";
	public static final String TAG_BEZEICHNUNG = "OrganisationseinheitenBEZEICHNUNG";

	@Override
	public FormData parseFormData(FormData formData) {
		var zustaendigeStelleMap = getZustaendigeStelle(formData);
		var organisationseinheitenID = getOrganisationseinheitenId(formData);
		var builder = ZustaendigeStelle.builder();

		if (Objects.isNull(zustaendigeStelleMap) && Objects.isNull(organisationseinheitenID)) {
			return formData;
		} else if (zustaendigeStelleMap != null) {
			organisationseinheitenID = (String) zustaendigeStelleMap.get(ORGANISATIONSEINHEITEN_ID);
			builder
					.email((String) zustaendigeStelleMap.get(EMAIL))
					.bezeichnung((String) zustaendigeStelleMap.get(TAG_BEZEICHNUNG));
		}
		var zustaendigeStelle = builder
				.organisationseinheitenId(organisationseinheitenID)
				.build();

		return formData.toBuilder().formData(addMetaDataFlag(formData)).zustaendigeStelle(zustaendigeStelle).build();
	}

	String getOrganisationseinheitenId(FormData formData) {
		return (String) formData.getFormData().get(ORGANISATIONSEINHEITEN_ID);
	}

	Map<String, Object> addMetaDataFlag(FormData formData) {
		var editableFormData = new LinkedHashMap<>(formData.getFormData());

		Optional.ofNullable(getZustaendigeStelle(formData))
				.map(LinkedHashMap::new)
				.ifPresent(zustaendigeStelle -> {
					zustaendigeStelle.put(KOP_CONTROLDATA_NODENAME, buildControlDataMap());
					editableFormData.put(ZUSTAENDIGESTELLE, zustaendigeStelle);
				});

		return Collections.unmodifiableMap(editableFormData);
	}

	@SuppressWarnings("unchecked")
	Map<String, Object> getZustaendigeStelle(FormData formData) {
		return (Map<String, Object>) formData.getFormData().get(ZUSTAENDIGESTELLE);
	}
}