/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;
import de.ozgcloud.eingang.semantik.enginebased.ServiceKontoBuildHelper;
import de.ozgcloud.eingang.semantik.enginebased.afm.AfmHeaderMapper;

class AfmHeaderMapperTest {

	@Spy
	@InjectMocks
	private final AfmHeaderMapper mapper = new AfmHeaderMapper();
	@Mock
	private ServiceKontoBuildHelper serviceKontoBuildHelper;

	@DisplayName("Parse form data")
	@Nested
	class TestParseFormData {

		@DisplayName("map form header")
		@Nested
		class TestMapFormHeader {

			private static final FormData FORM_DATA = FormData.builder().formData(AfmHeaderTestFactory.createFormDataMap()).build();

			@DisplayName("fields")
			@Nested
			class TestFields {

				@Test
				void shouldMapRequestId() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getRequestId()).isEqualTo(AfmHeaderTestFactory.ID);
				}

				@Test
				void shouldMapCreatedAt() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getCreatedAt()).isEqualTo(AfmHeaderTestFactory.TIMESTAMP);
				}

				@Test
				void shouldMapFormId() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getFormId()).isEqualTo(AfmHeaderTestFactory.FORM_ID);
				}

				@Test
				void shouldMapFormName() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getFormName()).isEqualTo(AfmHeaderTestFactory.FORM);
				}

				@Test
				void shouldMapSender() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getSender()).isEqualTo(AfmHeaderTestFactory.SENDER);
				}

				@Test
				void shouldSetFormEngineName() {
					var parsedFormData = parseFormData();

					assertThat(parsedFormData.getHeader().getFormEngineName()).isEqualTo(AfmHeaderMapper.AFM_FORMENGINE_NAME);
				}

				@DisplayName("service konto")
				@Nested
				class TestServiceKonto {

					@Test
					void shouldCallBuildServiceKontoIfPresent() {
						parseFormData();

						verify(serviceKontoBuildHelper).buildServiceKonto(any(), eq(FORM_DATA));
					}

					@Test
					void shouldNotCallBuildServiceKontoIfNotExists() {
						mapper.parseFormData(FormDataUtils.from(FORM_DATA).remove(AfmHeaderMapper.POSTFACH_NAME_ID).build());

						verify(serviceKontoBuildHelper, never()).buildServiceKonto(any(), any());
					}
				}

				private FormData parseFormData() {
					return mapper.parseFormData(FORM_DATA);
				}
			}
		}

		@DisplayName("remove mapped data")
		@Nested
		class TestRemoveMappedData {

			private final FormData formData = FormData.builder().formData(AfmHeaderTestFactory.createFormDataMap()).build();

			@Test
			void shouldRemoveHeader() {
				var parsedFormData = parseFormData();

				assertThat(parsedFormData.getFormData().get(AfmHeaderMapper.HEADER_FIELD)).isNull();
			}

			@Test
			void shouldRemoveRestResponsName() {
				var parsedFormData = parseFormData();

				assertThat(parsedFormData.getFormData().get(ServiceKontoBuildHelper.REST_RESPONSE_NAME)).isNull();
			}

			private FormData parseFormData() {
				return mapper.parseFormData(formData);
			}
		}
	}
}