# Intelliform Adapter

Adapter zum empfangen von Formulardaten von einem IntellForm basierten Formularserveri, zum Beispiel iAFM (integriertes Antrags- und Formularmanagement).

## Routingkonfiguration

### Pluto Instanzen

Für alle Pluto-Instanzen, die von dem Adapter erreichbar sein sollen, muss in das _Environment_ ein Eintrag mit dem GRPC-Service ergänzt werden:

```yaml
grpc.client.pluto-*pluto-name*.address:*url und port*
grpc.client.pluto-*pluto-name*.negotiationType: PLAINTEXT
```

### Fallback Strategy

Die Fallback Stratey steuert wie mit Eingängen umgegangen werden soll, für die keine passende Pluto-Instanze gefunden werden konnte.

Folgende Optionen stehen zur Verfügung:

- **DENY** der Antrag wird mit einer Fehlermeldung abgelehnt. Dies funktioniert nur, solange die Abarbeitung synchron erfolgt.

- **FUNDSTELLE** der Antrag wird an eine zentrale Fundstelle weitergeleitet. Dafür muss der Name der Pluto-Instanze, die als Fundstelle fungiert, eingetragen werden.

### Routing Strategy

Die Routing Strategy steuert wie das Routing konfiguriert wird und wieviele Pluto-Instanzen berücksichtigt werden können.

Folgende Optionen stehen zur Verfügung:

- **MULTI** es kann an beliebig viele Pluto-Instanzen geroutet werden. Dafür muss in der Environment ein Mapping der Organisationseinheit-Id auf den Namen einer Pluto-Instanz konfiguriert werden.

```yaml
kop.adapter.organisationseinheiten.*id*: *pluto-name*
```

- **SINGLE** es wird immer nur an eine Pluto-Instanze geroutet, der Name der Instanz ist im Feld 'Pluto Name' anzugeben.

### Beispielkonfiguration

```yaml
env:
  springProfiles: "oc, dev"
  grpc:
    - name: grpc_client_pluto-test_address
      value: "pluto.test:9090"
    - name: grpc_client_pluto-test_negotiationType
      value: PLAINTEXT
    - name: kop.adapter.organisationseinheiten.1357913579
      value: test
image:
  tag: snapshot-latest
imageCredentials:
  email: webmaster@ozg-sh.de
  password:  
  registry: docker.ozg-sh.de
  username: kop
ingress:
  host: kiel-afm.dev.by.kop-cloud.de
replicaCount: 2
resources:
  limits:
    cpu: 1
    memory: 1200Mi
  requests:
    cpu: 100m
    memory: 250Mi
global:
  cattle:
    clusterId: c-8g78g
    clusterName: ozg-dev
    systemDefaultRegistry: ""
  systemDefaultRegistry: ""
routing:
  fallbackStrategy: FUNDSTELLE
  fundstellePlutoName: kiel
  routingStrategy: MULTI
  targetPlutoName: kiel
```

### Benutzung beliebiger environment Werte

In jedem der Projekte kann man beliebige weitere environments setzen. Dazu muss man in der jeweiligen values.yaml unter env.customList ein name value Paar setzen:

```yaml
env:
  customList:
    - name: Dinge
      value: true
    - name: ...
      value: ...
```
